package kr.co.vividnext.sodalive.admin.content.curation

import kr.co.vividnext.sodalive.common.ApiResponse
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.PutMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("/admin/audio-content/curation")
@PreAuthorize("hasRole('ADMIN')")
class AdminContentCurationController(private val service: AdminContentCurationService) {
    @PostMapping
    fun createContentCuration(
        @RequestBody request: CreateContentCurationRequest
    ) = ApiResponse.ok(service.createContentCuration(request))

    @PutMapping
    fun updateContentCuration(
        @RequestBody request: UpdateContentCurationRequest
    ) = ApiResponse.ok(service.updateContentCuration(request))

    @PutMapping("/orders")
    fun updateContentCurationOrders(
        @RequestBody request: UpdateContentCurationOrdersRequest
    ) = ApiResponse.ok(service.updateContentCurationOrders(request.ids), "수정되었습니다.")

    @GetMapping
    fun getContentCurationList(
        @RequestParam tabId: Long
    ) = ApiResponse.ok(service.getContentCurationList(tabId = tabId))

    @GetMapping("/items")
    fun getCurationItems(
        @RequestParam curationId: Long
    ) = ApiResponse.ok(service.getCurationItem(curationId = curationId))

    @GetMapping("/search/content")
    fun searchCurationContentItem(
        @RequestParam curationId: Long,
        @RequestParam searchWord: String
    ) = ApiResponse.ok(service.searchCurationContentItem(curationId, searchWord))

    @GetMapping("/search/series")
    fun searchCurationSeriesItem(
        @RequestParam curationId: Long,
        @RequestParam searchWord: String
    ) = ApiResponse.ok(service.searchCurationSeriesItem(curationId, searchWord))

    @PostMapping("/add/item")
    fun addItemToCuration(
        @RequestBody request: AddItemToCurationRequest
    ) = ApiResponse.ok(service.addItemToCuration(request), "큐레이션 아이템을 등록했습니다.")

    @PutMapping("/remove/item")
    fun removeItemInCuration(
        @RequestBody request: RemoveItemInCurationRequest
    ) = ApiResponse.ok(service.removeItemInCuration(request), "큐레이션 아이템을 제거했습니다.")

    @PutMapping("/orders/item")
    fun updateItemInCurationOrders(
        @RequestBody request: UpdateCurationItemOrdersRequest
    ) = ApiResponse.ok(service.updateItemInCurationOrders(request), "수정되었습니다.")
}
