package kr.co.vividnext.sodalive.admin.content.series.recommend

import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.content.main.tab.QRecommendSeries.recommendSeries
import kr.co.vividnext.sodalive.content.main.tab.RecommendSeries
import kr.co.vividnext.sodalive.creator.admin.content.series.QSeries.series
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.jpa.repository.JpaRepository

interface AdminRecommendSeriesRepository :
    JpaRepository<RecommendSeries, Long>,
    AdminRecommendSeriesQueryRepository

interface AdminRecommendSeriesQueryRepository {
    fun getRecommendSeriesList(isFree: Boolean): List<GetAdminRecommendSeriesListResponse>
}

class AdminRecommendSeriesQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory,

    @Value("\${cloud.aws.cloud-front.host}")
    private val imageHost: String
) : AdminRecommendSeriesQueryRepository {
    override fun getRecommendSeriesList(isFree: Boolean): List<GetAdminRecommendSeriesListResponse> {
        return queryFactory
            .select(
                QGetAdminRecommendSeriesListResponse(
                    recommendSeries.id,
                    series.id,
                    series.title,
                    recommendSeries.imagePath.prepend("/").prepend(imageHost)
                )
            )
            .from(recommendSeries)
            .innerJoin(recommendSeries.series, series)
            .where(
                recommendSeries.isActive.isTrue
                    .and(series.isActive.isTrue)
                    .and(recommendSeries.isFree.eq(isFree))
            )
            .orderBy(recommendSeries.orders.asc())
            .fetch()
    }
}
