package kr.co.vividnext.sodalive.admin.event.banner

import com.querydsl.core.types.dsl.DateTimePath
import com.querydsl.core.types.dsl.Expressions
import com.querydsl.core.types.dsl.StringExpression
import com.querydsl.core.types.dsl.StringPath
import com.querydsl.core.types.dsl.StringTemplate
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.event.Event
import kr.co.vividnext.sodalive.event.QEvent.event
import org.springframework.beans.factory.annotation.Value
import org.springframework.data.jpa.repository.JpaRepository
import java.time.LocalDateTime

interface AdminEventBannerRepository : JpaRepository<Event, Long>, AdminEventBannerQueryRepository

interface AdminEventBannerQueryRepository {
    fun getEventList(): List<GetAdminEventResponse>
}

class AdminEventBannerQueryRepositoryImpl(
    private val queryFactory: JPAQueryFactory,
    @Value("\${cloud.aws.cloud-front.host}")
    private val cloudFrontHost: String
) : AdminEventBannerQueryRepository {
    override fun getEventList(): List<GetAdminEventResponse> {
        val now = LocalDateTime.now()
        val where = event.isActive.isTrue
            .and(event.endDate.goe(now))

        return queryFactory
            .select(
                QGetAdminEventResponse(
                    event.id,
                    event.title,
                    getProcessedUrlExpression(event.thumbnailImage),
                    getProcessedUrlExpression(event.detailImage),
                    getProcessedUrlExpression(event.popupImage),
                    getFormattedDate(event.startDate),
                    getFormattedDate(event.endDate),
                    event.link,
                    event.isAdult,
                    event.isPopup
                )
            )
            .from(event)
            .where(where)
            .orderBy(event.id.desc())
            .fetch()
    }

    private fun getFormattedDate(dateTimePath: DateTimePath<LocalDateTime>): StringTemplate {
        return Expressions.stringTemplate(
            "DATE_FORMAT({0}, {1})",
            Expressions.dateTimeTemplate(
                LocalDateTime::class.java,
                "CONVERT_TZ({0},{1},{2})",
                dateTimePath,
                "UTC",
                "Asia/Seoul"
            ),
            "%Y-%m-%d %H:%i"
        )
    }

    private fun getProcessedUrlExpression(path: StringPath): StringExpression {
        return Expressions.stringTemplate(
            "CASE WHEN {0} LIKE '$cloudFrontHost%' THEN {0} ELSE CONCAT('$cloudFrontHost/', {0}) END",
            path
        )
    }
}
