package kr.co.vividnext.sodalive.admin.marketing

import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.marketing.AdMediaPartnerRepository
import kr.co.vividnext.sodalive.marketing.AdMediaPartnerType
import org.springframework.data.repository.findByIdOrNull
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import java.net.URLEncoder
import java.nio.charset.StandardCharsets

@Service
class AdminAdMediaPartnerService(private val repository: AdMediaPartnerRepository) {
    @Transactional
    fun createMediaPartner(request: CreateAdMediaPartnerRequest) {
        val mediaPartner = request.toEntity()
        repository.save(mediaPartner)
    }

    @Transactional
    fun updateMediaPartner(request: UpdateAdMediaPartnerRequest) {
        val entity = repository.findByIdOrNull(request.id)
            ?: throw SodaException("잘못된 접근입니다")

        if (request.mediaGroup != null) {
            entity.mediaGroup = request.mediaGroup
        }

        if (request.pid != null) {
            entity.pid = request.pid
        }

        if (request.pidName != null) {
            entity.pidName = request.pidName
        }

        if (request.type != null) {
            entity.type = request.type
        }

        if (request.utmSource != null) {
            entity.utmSource = request.utmSource
        }

        if (request.utmMedium != null) {
            entity.utmMedium = request.utmMedium
        }

        if (request.isActive != null) {
            entity.isActive = request.isActive
        }
    }

    fun getMediaPartnerList(offset: Long, limit: Long): GetAdminAdMediaPartnerResponse {
        val totalCount = repository.getMediaPartnerListTotalCount()
        val items = repository.getMediaPartnerList(offset, limit)
            .map {
                val deepLinkValue = when (it.type) {
                    AdMediaPartnerType.SERIES -> "series"
                    AdMediaPartnerType.CONTENT -> "content"
                    AdMediaPartnerType.LIVE -> "live"
                    AdMediaPartnerType.CHANNEL -> "channel"
                    AdMediaPartnerType.MAIN -> "main"
                }

                val params = mapOf(
                    "af_dp" to "voiceon://",
                    "deep_link_value" to deepLinkValue,
                    "deep_link_sub1" to it.pid,
                    "deep_link_sub2" to it.utmSource,
                    "deep_link_sub3" to it.utmMedium,
                    "deep_link_sub4" to it.pidName,
                    "utm_source" to it.utmSource,
                    "utm_medium" to it.utmMedium,
                    "utm_campaign" to it.pid
                )
                val link = createUrl(params = params)

                it.link = link
                it
            }

        return GetAdminAdMediaPartnerResponse(
            totalCount = totalCount,
            items = items
        )
    }

    private fun createUrl(params: Map<String, String>): String {
        val encodedParams = params.map { (key, value) ->
            "${URLEncoder.encode(key, StandardCharsets.UTF_8.toString())}=" +
                URLEncoder.encode(value, StandardCharsets.UTF_8.toString())
        }.joinToString("&")

        return "$ONE_LINK_HOST?$encodedParams"
    }

    companion object {
        private const val ONE_LINK_HOST = "https://voiceon.onelink.me/RkTm"
    }
}
