package kr.co.vividnext.sodalive.content.category

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.member.Member
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.DeleteMapping
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PathVariable
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.PutMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("/category")
class CategoryController(private val service: CategoryService) {
    @PostMapping
    @PreAuthorize("hasRole('CREATOR')")
    fun createCategory(
        @RequestBody request: CreateCategoryRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.createCategory(request = request, member = member))
    }

    @PutMapping
    @PreAuthorize("hasRole('CREATOR')")
    fun modifyCategory(
        @RequestBody request: ModifyCategoryRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.modifyCategory(request = request, member = member))
    }

    @PutMapping("/orders")
    @PreAuthorize("hasRole('CREATOR')")
    fun updateCategoryOrders(
        @RequestBody request: UpdateCategoryOrdersRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.updateCategoryOrders(request = request, member = member))
    }

    @DeleteMapping("/{id}")
    @PreAuthorize("hasRole('CREATOR')")
    fun deleteCategory(
        @PathVariable("id") categoryId: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.deleteCategory(categoryId = categoryId, member = member))
    }

    @GetMapping
    fun getCategoryList(
        @RequestParam creatorId: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.getCategoryList(creatorId = creatorId, memberId = member.id!!))
    }
}
