package kr.co.vividnext.sodalive.content.main.tab.replay

import kr.co.vividnext.sodalive.content.AudioContentRepository
import kr.co.vividnext.sodalive.content.ContentType
import kr.co.vividnext.sodalive.content.main.GetAudioContentRankingItem
import kr.co.vividnext.sodalive.content.main.banner.AudioContentBannerService
import kr.co.vividnext.sodalive.content.main.curation.AudioContentCurationQueryRepository
import kr.co.vividnext.sodalive.content.main.tab.AudioContentMainTabRepository
import kr.co.vividnext.sodalive.content.main.tab.GetContentCurationResponse
import kr.co.vividnext.sodalive.event.EventService
import kr.co.vividnext.sodalive.member.Member
import kr.co.vividnext.sodalive.rank.RankingService
import org.springframework.stereotype.Service

@Service
class AudioContentMainTabLiveReplayService(
    private val repository: AudioContentMainTabRepository,
    private val bannerService: AudioContentBannerService,
    private val contentRepository: AudioContentRepository,
    private val rankingService: RankingService,
    private val eventService: EventService,
    private val curationRepository: AudioContentCurationQueryRepository
) {
    fun fetchData(
        isAdultContentVisible: Boolean,
        contentType: ContentType,
        member: Member
    ): GetContentMainTabLiveReplayResponse {
        val isAdult = member.auth != null && isAdultContentVisible
        val memberId = member.id!!
        val theme = "다시듣기"
        val tabId = 6L

        val contentBannerList = bannerService.getBannerList(
            tabId = tabId,
            memberId = memberId,
            isAdult = isAdult
        )

        val newLiveReplayContentList = contentRepository.findByTheme(
            memberId = memberId,
            theme = listOf(theme),
            isAdult = isAdult,
            contentType = contentType,
            limit = 10
        )

        val creatorList = repository.findCreatorByThemeContent(
            memberId = memberId,
            theme = theme,
            minCount = 4,
            isAdult = isAdult,
            contentType = contentType
        )

        val salesCountRankContentList = if (creatorList.isNotEmpty()) {
            rankingService.fetchCreatorContentBySalesCountTop4(
                creatorId = creatorList[0].creatorId,
                isAdult = isAdult,
                contentType = contentType,
                theme = theme
            )
        } else {
            emptyList()
        }

        val eventBannerList = eventService.getEventList(isAdult = isAdult)

        val curationList = curationRepository.findByContentMainTabId(tabId = tabId, isAdult = isAdult)
            .map {
                GetContentCurationResponse(
                    title = it.title,
                    items = contentRepository.findAudioContentByCurationIdV2(
                        curationId = it.id!!,
                        memberId = memberId,
                        isAdult = isAdult,
                        contentType = contentType
                    )
                )
            }
            .filter { it.items.isNotEmpty() }

        return GetContentMainTabLiveReplayResponse(
            contentBannerList = contentBannerList,
            newLiveReplayContentList = newLiveReplayContentList,
            creatorList = creatorList,
            salesCountRankContentList = salesCountRankContentList,
            eventBannerList = eventBannerList,
            curationList = curationList
        )
    }

    fun getPopularContentByCreator(
        creatorId: Long,
        isAdult: Boolean,
        contentType: ContentType
    ): List<GetAudioContentRankingItem> {
        return rankingService.fetchCreatorContentBySalesCountTop4(
            creatorId = creatorId,
            isAdult = isAdult,
            contentType = contentType,
            theme = "다시듣기"
        )
    }
}
